@@ -24,11 +24,13 @@
 #include <stddef.h>
 #include <stdint.h>
 
+#include <cstddef>
 #include <optional>
 
 #include "absl/status/statusor.h"
 #include "absl/strings/string_view.h"
 #include "src/core/call/metadata_batch.h"
+#include "src/core/channelz/property_list.h"
 #include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/channel/channel_fwd.h"
 #include "src/core/lib/channel/promise_based_filter.h"
@@ -91,6 +93,17 @@ class ChannelCompression {
       bool is_client, MessageHandle message, DecompressArgs args,
       CallTracerInterface* call_tracer) const;
 
+  channelz::PropertyList ChannelzProperties() const {
+    return channelz::PropertyList()
+        .Set("max_recv_size", max_recv_size_)
+        .Set("default_compression_algorithm",
+             CompressionAlgorithmAsString(default_compression_algorithm_))
+        .Set("enabled_compression_algorithms",
+             enabled_compression_algorithms_.ToString())
+        .Set("enable_compression", enable_compression_)
+        .Set("enable_decompression", enable_decompression_);
+  }
+
  private:
   // Max receive message length, if set.
   std::optional<uint32_t> max_recv_size_;
@@ -106,7 +119,8 @@ class ChannelCompression {
 };
 
 class ClientCompressionFilter final
-    : public ImplementChannelFilter<ClientCompressionFilter> {
+    : public ImplementChannelFilter<ClientCompressionFilter>,
+      public channelz::DataSource {
  public:
   static const grpc_channel_filter kFilter;
 
@@ -116,7 +130,16 @@ class ClientCompressionFilter final
       const ChannelArgs& args, ChannelFilter::Args filter_args);
 
   explicit ClientCompressionFilter(const ChannelArgs& args)
-      : compression_engine_(args) {}
+      : channelz::DataSource(args.GetObjectRef<channelz::BaseNode>()),
+        compression_engine_(args) {
+    SourceConstructed();
+  }
+  ~ClientCompressionFilter() override { SourceDestructing(); }
+
+  void AddData(channelz::DataSink sink) override {
+    sink.AddData("clientCompressionFilter",
+                 compression_engine_.ChannelzProperties());
+  }
 
   // Construct a promise for one call.
   class Call {
@@ -148,7 +171,8 @@ class ClientCompressionFilter final
 };
 
 class ServerCompressionFilter final
-    : public ImplementChannelFilter<ServerCompressionFilter> {
+    : public ImplementChannelFilter<ServerCompressionFilter>,
+      public channelz::DataSource {
  public:
   static const grpc_channel_filter kFilter;
 
@@ -158,7 +182,16 @@ class ServerCompressionFilter final
       const ChannelArgs& args, ChannelFilter::Args filter_args);
 
   explicit ServerCompressionFilter(const ChannelArgs& args)
-      : compression_engine_(args) {}
+      : channelz::DataSource(args.GetObjectRef<channelz::BaseNode>()),
+        compression_engine_(args) {
+    SourceConstructed();
+  }
+  ~ServerCompressionFilter() override { SourceDestructing(); }
+
+  void AddData(channelz::DataSink sink) override {
+    sink.AddData("serverCompressionFilter",
+                 compression_engine_.ChannelzProperties());
+  }
 
   // Construct a promise for one call.
   class Call {
