@@ -37,10 +37,12 @@ namespace grpc_core {
 
 std::optional<std::string> GetEnv(const char* name) {
   char* result = nullptr;
-#if __GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ >= 17)
+#if (__GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ >= 17)) && \
+    !defined(GRPC_FORCE_UNSECURE_GETENV)
   result = secure_getenv(name);
 #else
   result = getenv(name);
+
 #endif
   if (result == nullptr) return std::nullopt;
   return result;
