@@ -65,12 +65,13 @@ namespace grpc {
 namespace internal {
 
 template <typename UnrefBehavior>
-class OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
+class OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer<
     UnrefBehavior>::TcpCallTracer : public grpc_core::TcpCallTracer {
  public:
   explicit TcpCallTracer(
-      grpc_core::RefCountedPtr<OpenTelemetryPluginImpl::ClientCallTracer::
-                                   CallAttemptTracer<UnrefBehavior>>
+      grpc_core::RefCountedPtr<
+          OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer<
+              UnrefBehavior>>
           call_attempt_tracer)
       : call_attempt_tracer_(std::move(call_attempt_tracer)) {
     // Take a ref on the call if tracing is enabled, since TCP traces might
@@ -78,7 +79,8 @@ class OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
     call_attempt_tracer_->parent_->arena_
         ->template GetContext<grpc_core::Call>()
         ->InternalRef(
-            "OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer::"
+            "OpenTelemetryPluginImpl::ClientCallTracerInterface::"
+            "CallAttemptTracer::"
             "TcpCallTracer");
   }
 
@@ -89,7 +91,8 @@ class OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
     // reset before unreffing the call.
     call_attempt_tracer_.reset();
     arena->template GetContext<grpc_core::Call>()->InternalUnref(
-        "OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer::~"
+        "OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer:"
+        ":~"
         "TcpCallTracer");
   }
 
@@ -105,18 +108,19 @@ class OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
   }
 
  private:
-  grpc_core::RefCountedPtr<OpenTelemetryPluginImpl::ClientCallTracer::
+  grpc_core::RefCountedPtr<OpenTelemetryPluginImpl::ClientCallTracerInterface::
                                CallAttemptTracer<UnrefBehavior>>
       call_attempt_tracer_;
 };
 
 //
-// OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer
+// OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer
 //
 template <typename UnrefBehavior>
-OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<UnrefBehavior>::
-    CallAttemptTracer(const OpenTelemetryPluginImpl::ClientCallTracer* parent,
-                      uint64_t attempt_num, bool is_transparent_retry)
+OpenTelemetryPluginImpl::ClientCallTracerInterface::
+    CallAttemptTracer<UnrefBehavior>::CallAttemptTracer(
+        OpenTelemetryPluginImpl::ClientCallTracerInterface* const parent,
+        uint64_t attempt_num, bool is_transparent_retry)
     : parent_(parent), start_time_(absl::Now()) {
   if (parent_->otel_plugin_->client_.attempt.started != nullptr) {
     std::array<std::pair<absl::string_view, absl::string_view>, 2>
@@ -148,7 +152,7 @@ OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<UnrefBehavior>::
 }
 
 template <typename UnrefBehavior>
-OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
+OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer<
     UnrefBehavior>::~CallAttemptTracer<UnrefBehavior>() {
   if (span_ != nullptr) {
     span_->End();
@@ -156,7 +160,7 @@ OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
 }
 
 template <typename UnrefBehavior>
-void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
+void OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer<
     UnrefBehavior>::RecordReceivedInitialMetadata(grpc_metadata_batch*
                                                       recv_initial_metadata) {
   if (recv_initial_metadata != nullptr &&
@@ -169,7 +173,7 @@ void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
 }
 
 template <typename UnrefBehavior>
-void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
+void OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer<
     UnrefBehavior>::RecordSendInitialMetadata(grpc_metadata_batch*
                                                   send_initial_metadata) {
   parent_->scope_config_->active_plugin_options_view().ForEach(
@@ -183,10 +187,12 @@ void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
       },
       parent_->otel_plugin_);
   if (span_ != nullptr) {
-    GrpcTextMapCarrier carrier(send_initial_metadata);
-    opentelemetry::context::Context context;
-    context = opentelemetry::trace::SetSpan(context, span_);
-    parent_->otel_plugin_->text_map_propagator_->Inject(carrier, context);
+    if (parent_->otel_plugin_->text_map_propagator_ != nullptr) {
+      GrpcTextMapCarrier carrier(send_initial_metadata);
+      opentelemetry::context::Context context;
+      context = opentelemetry::trace::SetSpan(context, span_);
+      parent_->otel_plugin_->text_map_propagator_->Inject(carrier, context);
+    }
     if (IsSampled()) {
       parent_->arena_->GetContext<grpc_core::Call>()->set_traced(true);
     }
@@ -194,7 +200,7 @@ void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
 }
 
 template <typename UnrefBehavior>
-void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
+void OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer<
     UnrefBehavior>::RecordSendMessage(const grpc_core::Message& send_message) {
   if (span_ != nullptr) {
     std::array<std::pair<opentelemetry::nostd::string_view,
@@ -208,7 +214,7 @@ void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
 }
 
 template <typename UnrefBehavior>
-void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
+void OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer<
     UnrefBehavior>::RecordSendCompressedMessage(const grpc_core::Message&
                                                     send_compressed_message) {
   if (span_ != nullptr) {
@@ -226,7 +232,7 @@ void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
 }
 
 template <typename UnrefBehavior>
-void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
+void OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer<
     UnrefBehavior>::RecordReceivedMessage(const grpc_core::Message&
                                               recv_message) {
   if (span_ != nullptr) {
@@ -249,7 +255,7 @@ void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
 }
 
 template <typename UnrefBehavior>
-void OpenTelemetryPluginImpl::ClientCallTracer::
+void OpenTelemetryPluginImpl::ClientCallTracerInterface::
     CallAttemptTracer<UnrefBehavior>::RecordReceivedDecompressedMessage(
         const grpc_core::Message& recv_decompressed_message) {
   if (span_ != nullptr) {
@@ -267,7 +273,7 @@ void OpenTelemetryPluginImpl::ClientCallTracer::
 }
 
 template <typename UnrefBehavior>
-void OpenTelemetryPluginImpl::ClientCallTracer::
+void OpenTelemetryPluginImpl::ClientCallTracerInterface::
     CallAttemptTracer<UnrefBehavior>::RecordReceivedTrailingMetadata(
         absl::Status status, grpc_metadata_batch* recv_trailing_metadata,
         const grpc_transport_stream_stats* transport_stream_stats) {
@@ -310,6 +316,12 @@ void OpenTelemetryPluginImpl::ClientCallTracer::
     parent_->otel_plugin_->client_.attempt.rcvd_total_compressed_message_size
         ->Record(incoming_bytes, labels, opentelemetry::context::Context{});
   }
+  if (parent_->otel_plugin_->client_.call.retry_delay != nullptr) {
+    grpc_core::MutexLock lock(&parent_->mu_);
+    if (--parent_->num_active_attempts_ == 0) {
+      parent_->time_at_last_attempt_end_ = absl::Now();
+    }
+  }
   if (span_ != nullptr) {
     if (status.ok()) {
       span_->SetStatus(opentelemetry::trace::StatusCode::kOk);
@@ -321,31 +333,31 @@ void OpenTelemetryPluginImpl::ClientCallTracer::
 }
 
 template <typename UnrefBehavior>
-void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
+void OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer<
     UnrefBehavior>::RecordIncomingBytes(const TransportByteSize&
                                             transport_byte_size) {
   incoming_bytes_.fetch_add(transport_byte_size.data_bytes);
 }
 
 template <typename UnrefBehavior>
-void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
+void OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer<
     UnrefBehavior>::RecordOutgoingBytes(const TransportByteSize&
                                             transport_byte_size) {
   outgoing_bytes_.fetch_add(transport_byte_size.data_bytes);
 }
 
 template <typename UnrefBehavior>
-void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
+void OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer<
     UnrefBehavior>::RecordCancel(absl::Status /*cancel_error*/) {}
 
 template <typename UnrefBehavior>
-void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
+void OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer<
     UnrefBehavior>::RecordEnd() {
   this->Unref(DEBUG_LOCATION, "RecordEnd");
 }
 
 template <typename UnrefBehavior>
-void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
+void OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer<
     UnrefBehavior>::RecordAnnotation(absl::string_view annotation) {
   if (span_ != nullptr) {
     span_->AddEvent(AbslStringViewToNoStdStringView(annotation));
@@ -353,13 +365,13 @@ void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
 }
 
 template <typename UnrefBehavior>
-void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
+void OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer<
     UnrefBehavior>::RecordAnnotation(const Annotation& /*annotation*/) {
   // Not implemented
 }
 
 template <typename UnrefBehavior>
-void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
+void OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer<
     UnrefBehavior>::RecordAnnotation(absl::string_view annotation,
                                      absl::Time time) {
   if (span_ != nullptr) {
@@ -369,8 +381,9 @@ void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
 }
 
 template <typename UnrefBehavior>
-std::shared_ptr<grpc_core::TcpCallTracer> OpenTelemetryPluginImpl::
-    ClientCallTracer::CallAttemptTracer<UnrefBehavior>::StartNewTcpTrace() {
+std::shared_ptr<grpc_core::TcpCallTracer>
+OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer<
+    UnrefBehavior>::StartNewTcpTrace() {
   if (span_ != nullptr) {
     return std::make_shared<TcpCallTracer>(
         this->Ref(DEBUG_LOCATION, "StartNewTcpTrace"));
@@ -379,7 +392,7 @@ std::shared_ptr<grpc_core::TcpCallTracer> OpenTelemetryPluginImpl::
 }
 
 template <typename UnrefBehavior>
-void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
+void OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer<
     UnrefBehavior>::SetOptionalLabel(OptionalLabelKey key,
                                      grpc_core::RefCountedStringValue value) {
   CHECK(key < OptionalLabelKey::kSize);
@@ -387,7 +400,7 @@ void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
 }
 
 template <typename UnrefBehavior>
-void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
+void OpenTelemetryPluginImpl::ClientCallTracerInterface::CallAttemptTracer<
     UnrefBehavior>::PopulateLabelInjectors(grpc_metadata_batch* metadata) {
   parent_->scope_config_->active_plugin_options_view().ForEach(
       [&](const InternalOpenTelemetryPluginOption& plugin_option,
@@ -403,10 +416,10 @@ void OpenTelemetryPluginImpl::ClientCallTracer::CallAttemptTracer<
 }
 
 //
-// OpenTelemetryPluginImpl::ClientCallTracer
+// OpenTelemetryPluginImpl::ClientCallTracerInterface
 //
 
-OpenTelemetryPluginImpl::ClientCallTracer::ClientCallTracer(
+OpenTelemetryPluginImpl::ClientCallTracerInterface::ClientCallTracerInterface(
     const grpc_core::Slice& path, grpc_core::Arena* arena,
     bool registered_method, OpenTelemetryPluginImpl* otel_plugin,
     std::shared_ptr<OpenTelemetryPluginImpl::ClientScopeConfig> scope_config)
@@ -440,14 +453,41 @@ OpenTelemetryPluginImpl::ClientCallTracer::ClientCallTracer(
   }
 }
 
-OpenTelemetryPluginImpl::ClientCallTracer::~ClientCallTracer() {
+OpenTelemetryPluginImpl::ClientCallTracerInterface::
+    ~ClientCallTracerInterface() {
+  std::array<std::pair<opentelemetry::nostd::string_view,
+                       opentelemetry::common::AttributeValue>,
+             2>
+      attributes = {
+          std::pair(AbslStringViewToNoStdStringView(OpenTelemetryMethodKey()),
+                    opentelemetry::common::AttributeValue(
+                        AbslStringViewToNoStdStringView(MethodForStats()))),
+          std::pair(AbslStringViewToNoStdStringView(OpenTelemetryTargetKey()),
+                    opentelemetry::common::AttributeValue(
+                        AbslStringViewToNoStdStringView(
+                            scope_config_->filtered_target())))};
+  if (otel_plugin_->client_.call.retries != nullptr && retries_ > 1) {
+    otel_plugin_->client_.call.retries->Record(
+        retries_ - 1, attributes, opentelemetry::context::Context{});
+  }
+  if (otel_plugin_->client_.call.transparent_retries != nullptr &&
+      transparent_retries_ != 0) {
+    otel_plugin_->client_.call.transparent_retries->Record(
+        transparent_retries_, attributes, opentelemetry::context::Context{});
+  }
+  if (otel_plugin_->client_.call.retry_delay != nullptr &&
+      retry_delay_ != absl::ZeroDuration() && retries_ > 1) {
+    otel_plugin_->client_.call.retry_delay->Record(
+        absl::ToDoubleSeconds(retry_delay_), attributes,
+        opentelemetry::context::Context{});
+  }
   if (span_ != nullptr) {
     span_->End();
   }
 }
 
-grpc_core::ClientCallTracer::CallAttemptTracer*
-OpenTelemetryPluginImpl::ClientCallTracer::StartNewAttempt(
+grpc_core::ClientCallTracerInterface::CallAttemptTracer*
+OpenTelemetryPluginImpl::ClientCallTracerInterface::StartNewAttempt(
     bool is_transparent_retry) {
   // We allocate the first attempt on the arena and all subsequent attempts
   // on the heap, so that in the common case we don't require a heap
@@ -458,6 +498,10 @@ OpenTelemetryPluginImpl::ClientCallTracer::StartNewAttempt(
     grpc_core::MutexLock lock(&mu_);
     if (transparent_retries_ != 0 || retries_ != 0) {
       is_first_attempt = false;
+      if (otel_plugin_->client_.call.retry_delay != nullptr &&
+          num_active_attempts_ == 0 && !is_transparent_retry) {
+        retry_delay_ += absl::Now() - time_at_last_attempt_end_;
+      }
     }
     if (is_transparent_retry) {
       ++transparent_retries_;
@@ -465,6 +509,7 @@ OpenTelemetryPluginImpl::ClientCallTracer::StartNewAttempt(
       ++retries_;
     }
     attempt_num = retries_ - 1;  // Sequence starts at 0
+    ++num_active_attempts_;
   }
   if (is_first_attempt) {
     return arena_
@@ -476,8 +521,8 @@ OpenTelemetryPluginImpl::ClientCallTracer::StartNewAttempt(
                                                        is_transparent_retry);
 }
 
-absl::string_view OpenTelemetryPluginImpl::ClientCallTracer::MethodForStats()
-    const {
+absl::string_view
+OpenTelemetryPluginImpl::ClientCallTracerInterface::MethodForStats() const {
   absl::string_view method = absl::StripPrefix(path_.as_string_view(), "/");
   if (registered_method_ ||
       (otel_plugin_->generic_method_attribute_filter() != nullptr &&
@@ -487,14 +532,14 @@ absl::string_view OpenTelemetryPluginImpl::ClientCallTracer::MethodForStats()
   return "other";
 }
 
-void OpenTelemetryPluginImpl::ClientCallTracer::RecordAnnotation(
+void OpenTelemetryPluginImpl::ClientCallTracerInterface::RecordAnnotation(
     absl::string_view annotation) {
   if (span_ != nullptr) {
     span_->AddEvent(AbslStringViewToNoStdStringView(annotation));
   }
 }
 
-void OpenTelemetryPluginImpl::ClientCallTracer::RecordAnnotation(
+void OpenTelemetryPluginImpl::ClientCallTracerInterface::RecordAnnotation(
     const Annotation& /*annotation*/) {
   // Not implemented
 }
