@@ -18,7 +18,6 @@
 
 #include <grpc/support/alloc.h>
 #include <grpc/support/string_util.h>
-#include <gtest/gtest.h>
 
 #include <deque>
 #include <list>
@@ -27,8 +26,10 @@
 
 #include "absl/log/check.h"
 #include "gmock/gmock.h"
+#include "gtest/gtest.h"
 #include "src/core/lib/slice/slice_internal.h"
 #include "src/core/util/crash.h"
+#include "src/core/util/match.h"
 #include "test/core/test_util/test_config.h"
 #include "test/core/test_util/tls_utils.h"
 
@@ -36,6 +37,8 @@ namespace grpc_core {
 
 namespace testing {
 
+namespace {
+
 constexpr const char* kCertName1 = "cert_1_name";
 constexpr const char* kCertName2 = "cert_2_name";
 constexpr const char* kRootCert1Name = "root_cert_1_name";
@@ -51,6 +54,62 @@ constexpr const char* kIdentityCert2Contents = "identity_cert_2_contents";
 constexpr const char* kErrorMessage = "error_message";
 constexpr const char* kRootErrorMessage = "root_error_message";
 constexpr const char* kIdentityErrorMessage = "identity_error_message";
+constexpr absl::string_view kSpiffeBundleMapPath =
+    "test/core/credentials/transport/tls/test_data/spiffe/"
+    "client_spiffebundle.json";
+
+constexpr absl::string_view kSpiffeBundleMapPath2 =
+    "test/core/credentials/transport/tls/test_data/spiffe/test_bundles/"
+    "spiffebundle.json";
+
+SpiffeBundleMap GetRawSpiffeBundleMap() {
+  auto map = SpiffeBundleMap::FromFile(kSpiffeBundleMapPath);
+  CHECK(map.ok()) << map.status();
+  return *map;
+}
+
+std::shared_ptr<RootCertInfo> GetTestSpiffeBundleMap() {
+  return std::make_shared<RootCertInfo>(GetRawSpiffeBundleMap());
+}
+
+SpiffeBundleMap GetRawSpiffeBundleMap2() {
+  auto map = SpiffeBundleMap::FromFile(kSpiffeBundleMapPath2);
+  CHECK(map.ok()) << map.status();
+  return *map;
+}
+
+std::shared_ptr<RootCertInfo> GetTestSpiffeBundleMap2() {
+  return std::make_shared<RootCertInfo>(GetRawSpiffeBundleMap2());
+}
+
+std::shared_ptr<RootCertInfo> GetRootCert1() {
+  return std::make_shared<RootCertInfo>(kRootCert1Contents);
+}
+
+std::shared_ptr<RootCertInfo> GetRootCert2() {
+  return std::make_shared<RootCertInfo>(kRootCert2Contents);
+}
+
+MATCHER_P2(MatchesCredentialInfo, root_matcher, identity_matcher, "") {
+  bool ok = true;
+  ok &= ::testing::ExplainMatchResult(root_matcher, arg.root_cert_info,
+                                      result_listener);
+  ok &= ::testing::ExplainMatchResult(identity_matcher, arg.key_cert_pairs,
+                                      result_listener);
+  return ok;
+}
+
+MATCHER_P(EqRootCert, cert, "") {
+  return ::testing::ExplainMatchResult(
+      ::testing::Pointee(::testing::VariantWith<std::string>(cert)), arg,
+      result_listener);
+}
+
+MATCHER_P(EqSpiffeBundleMap, map, "") {
+  return ::testing::ExplainMatchResult(
+      ::testing::Pointee(::testing::VariantWith<SpiffeBundleMap>(map)), arg,
+      result_listener);
+}
 
 class GrpcTlsCertificateDistributorTest : public ::testing::Test {
  protected:
@@ -62,12 +121,15 @@ class GrpcTlsCertificateDistributorTest : public ::testing::Test {
   // CredentialInfo to the cert_update_queue of state_, and check in each test
   // if the status updates are correct.
   struct CredentialInfo {
-    std::string root_certs;
     PemKeyCertPairList key_cert_pairs;
-    CredentialInfo(std::string root, PemKeyCertPairList key_cert)
-        : root_certs(std::move(root)), key_cert_pairs(std::move(key_cert)) {}
+    std::shared_ptr<RootCertInfo> root_cert_info;
+    CredentialInfo(std::shared_ptr<RootCertInfo> roots,
+                   PemKeyCertPairList key_cert)
+        : key_cert_pairs(std::move(key_cert)),
+          root_cert_info(std::move(roots)) {}
+
     bool operator==(const CredentialInfo& other) const {
-      return root_certs == other.root_certs &&
+      return root_cert_info == other.root_cert_info &&
              key_cert_pairs == other.key_cert_pairs;
     }
   };
@@ -116,17 +178,17 @@ class GrpcTlsCertificateDistributorTest : public ::testing::Test {
     ~TlsCertificatesTestWatcher() override { state_->watcher = nullptr; }
 
     void OnCertificatesChanged(
-        std::optional<absl::string_view> root_certs,
+        std::shared_ptr<RootCertInfo> roots,
         std::optional<PemKeyCertPairList> key_cert_pairs) override {
-      std::string updated_root;
-      if (root_certs.has_value()) {
-        updated_root = std::string(*root_certs);
+      std::shared_ptr<RootCertInfo> updated_root;
+      if (roots != nullptr) {
+        updated_root = std::move(roots);
       }
       PemKeyCertPairList updated_identity;
       if (key_cert_pairs.has_value()) {
         updated_identity = std::move(*key_cert_pairs);
       }
-      state_->cert_update_queue.emplace_back(std::move(updated_root),
+      state_->cert_update_queue.emplace_back(updated_root,
                                              std::move(updated_identity));
     }
 
@@ -219,11 +281,10 @@ TEST_F(GrpcTlsCertificateDistributorTest, BasicCredentialBehaviors) {
   EXPECT_FALSE(distributor_.HasKeyCertPairs(kIdentityCert1Name));
   // After setting the certificates to the corresponding cert names, the
   // distributor should possess the corresponding certs.
-  distributor_.SetKeyMaterials(kRootCert1Name, kRootCert1Contents,
-                               std::nullopt);
+  distributor_.SetKeyMaterials(kRootCert1Name, GetRootCert1(), std::nullopt);
   EXPECT_TRUE(distributor_.HasRootCerts(kRootCert1Name));
   distributor_.SetKeyMaterials(
-      kIdentityCert1Name, std::nullopt,
+      kIdentityCert1Name, nullptr,
       MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents));
   EXPECT_TRUE(distributor_.HasKeyCertPairs(kIdentityCert1Name));
   // Querying a non-existing cert name should return false.
@@ -237,28 +298,28 @@ TEST_F(GrpcTlsCertificateDistributorTest, UpdateCredentialsOnAnySide) {
               ::testing::ElementsAre(CallbackStatus(kCertName1, true, true)));
   // SetKeyMaterials should trigger watcher's OnCertificatesChanged method.
   distributor_.SetKeyMaterials(
-      kCertName1, kRootCert1Contents,
+      kCertName1, GetRootCert1(),
       MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents));
   EXPECT_THAT(
       watcher_state_1->GetCredentialQueue(),
-      ::testing::ElementsAre(CredentialInfo(
-          kRootCert1Contents,
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(kRootCert1Contents),
           MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents))));
   // Set root certs should trigger watcher's OnCertificatesChanged again.
-  distributor_.SetKeyMaterials(kCertName1, kRootCert2Contents, std::nullopt);
+  distributor_.SetKeyMaterials(kCertName1, GetRootCert2(), std::nullopt);
   EXPECT_THAT(
       watcher_state_1->GetCredentialQueue(),
-      ::testing::ElementsAre(CredentialInfo(
-          kRootCert2Contents,
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(kRootCert2Contents),
           MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents))));
   // Set identity certs should trigger watcher's OnCertificatesChanged again.
   distributor_.SetKeyMaterials(
-      kCertName1, std::nullopt,
+      kCertName1, nullptr,
       MakeCertKeyPairs(kIdentityCert2PrivateKey, kIdentityCert2Contents));
   EXPECT_THAT(
       watcher_state_1->GetCredentialQueue(),
-      ::testing::ElementsAre(CredentialInfo(
-          kRootCert2Contents,
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(kRootCert2Contents),
           MakeCertKeyPairs(kIdentityCert2PrivateKey, kIdentityCert2Contents))));
   CancelWatch(watcher_state_1);
 }
@@ -278,32 +339,32 @@ TEST_F(GrpcTlsCertificateDistributorTest, SameIdentityNameDiffRootName) {
                                       kRootCert2Name, true, false)));
   // Push credential updates to kRootCert1Name and check if the status works as
   // expected.
-  distributor_.SetKeyMaterials(kRootCert1Name, kRootCert1Contents,
-                               std::nullopt);
+  distributor_.SetKeyMaterials(kRootCert1Name, GetRootCert1(), std::nullopt);
   // Check the updates are delivered to watcher 1.
   EXPECT_THAT(watcher_state_1->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(kRootCert1Contents, {})));
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  EqRootCert(kRootCert1Contents), PemKeyCertPairList())));
   // Push credential updates to kRootCert2Name.
-  distributor_.SetKeyMaterials(kRootCert2Name, kRootCert2Contents,
-                               std::nullopt);
+  distributor_.SetKeyMaterials(kRootCert2Name, GetRootCert2(), std::nullopt);
   // Check the updates are delivered to watcher 2.
   EXPECT_THAT(watcher_state_2->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(kRootCert2Contents, {})));
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  EqRootCert(kRootCert2Contents), PemKeyCertPairList())));
   // Push credential updates to kIdentityCert1Name and check if the status works
   // as expected.
   distributor_.SetKeyMaterials(
-      kIdentityCert1Name, std::nullopt,
+      kIdentityCert1Name, nullptr,
       MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents));
   // Check the updates are delivered to watcher 1 and watcher 2.
   EXPECT_THAT(
       watcher_state_1->GetCredentialQueue(),
-      ::testing::ElementsAre(CredentialInfo(
-          kRootCert1Contents,
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(kRootCert1Contents),
           MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents))));
   EXPECT_THAT(
       watcher_state_2->GetCredentialQueue(),
-      ::testing::ElementsAre(CredentialInfo(
-          kRootCert2Contents,
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(kRootCert2Contents),
           MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents))));
   // Cancel watcher 1.
   CancelWatch(watcher_state_1);
@@ -332,33 +393,34 @@ TEST_F(GrpcTlsCertificateDistributorTest, SameRootNameDiffIdentityName) {
                                       kIdentityCert2Name, false, true)));
   // Push credential updates to kRootCert1Name and check if the status works as
   // expected.
-  distributor_.SetKeyMaterials(kRootCert1Name, kRootCert1Contents,
-                               std::nullopt);
+  distributor_.SetKeyMaterials(kRootCert1Name, GetRootCert1(), std::nullopt);
   // Check the updates are delivered to watcher 1.
   EXPECT_THAT(watcher_state_1->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(kRootCert1Contents, {})));
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  EqRootCert(kRootCert1Contents), PemKeyCertPairList())));
   // Check the updates are delivered to watcher 2.
   EXPECT_THAT(watcher_state_2->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(kRootCert1Contents, {})));
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  EqRootCert(kRootCert1Contents), PemKeyCertPairList())));
   // Push credential updates to SetKeyMaterials.
   distributor_.SetKeyMaterials(
-      kIdentityCert1Name, std::nullopt,
+      kIdentityCert1Name, nullptr,
       MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents));
   // Check the updates are delivered to watcher 1.
   EXPECT_THAT(
       watcher_state_1->GetCredentialQueue(),
-      ::testing::ElementsAre(CredentialInfo(
-          kRootCert1Contents,
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(kRootCert1Contents),
           MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents))));
   // Push credential updates to kIdentityCert2Name.
   distributor_.SetKeyMaterials(
-      kIdentityCert2Name, std::nullopt,
+      kIdentityCert2Name, nullptr,
       MakeCertKeyPairs(kIdentityCert2PrivateKey, kIdentityCert2Contents));
   // Check the updates are delivered to watcher 2.
   EXPECT_THAT(
       watcher_state_2->GetCredentialQueue(),
-      ::testing::ElementsAre(CredentialInfo(
-          kRootCert1Contents,
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(kRootCert1Contents),
           MakeCertKeyPairs(kIdentityCert2PrivateKey, kIdentityCert2Contents))));
   // Cancel watcher 1.
   CancelWatch(watcher_state_1);
@@ -381,13 +443,13 @@ TEST_F(GrpcTlsCertificateDistributorTest,
   // Push credential updates to kCertName1 and check if the status works as
   // expected.
   distributor_.SetKeyMaterials(
-      kCertName1, kRootCert1Contents,
+      kCertName1, GetRootCert1(),
       MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents));
   // Check the updates are delivered to watcher 1.
   EXPECT_THAT(
       watcher_state_1->GetCredentialQueue(),
-      ::testing::ElementsAre(CredentialInfo(
-          kRootCert1Contents,
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(kRootCert1Contents),
           MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents))));
   // Cancel watcher 1.
   CancelWatch(watcher_state_1);
@@ -408,34 +470,36 @@ TEST_F(GrpcTlsCertificateDistributorTest,
   // Push credential updates to kCertName1 and check if the status works as
   // expected.
   distributor_.SetKeyMaterials(
-      kCertName1, kRootCert1Contents,
+      kCertName1, GetRootCert1(),
       MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents));
   // Check the updates are delivered to watcher 1.
   EXPECT_THAT(watcher_state_1->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(kRootCert1Contents, {})));
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  EqRootCert(kRootCert1Contents), PemKeyCertPairList())));
   // Check the updates are delivered to watcher 2.
   EXPECT_THAT(watcher_state_2->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(
-                  "", MakeCertKeyPairs(kIdentityCert1PrivateKey,
-                                       kIdentityCert1Contents))));
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  nullptr, MakeCertKeyPairs(kIdentityCert1PrivateKey,
+                                            kIdentityCert1Contents))));
   // Push root cert updates to kCertName1.
-  distributor_.SetKeyMaterials(kCertName1, kRootCert2Contents, std::nullopt);
+  distributor_.SetKeyMaterials(kCertName1, GetRootCert2(), std::nullopt);
   // Check the updates are delivered to watcher 1.
   EXPECT_THAT(watcher_state_1->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(kRootCert2Contents, {})));
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  EqRootCert(kRootCert2Contents), PemKeyCertPairList())));
   // Check the updates are not delivered to watcher 2.
   EXPECT_THAT(watcher_state_2->GetCredentialQueue(), ::testing::ElementsAre());
   // Push identity cert updates to kCertName1.
   distributor_.SetKeyMaterials(
-      kCertName1, std::nullopt,
+      kCertName1, nullptr,
       MakeCertKeyPairs(kIdentityCert2PrivateKey, kIdentityCert2Contents));
   // Check the updates are not delivered to watcher 1.
   EXPECT_THAT(watcher_state_1->GetCredentialQueue(), ::testing::ElementsAre());
   // Check the updates are delivered to watcher 2.
   EXPECT_THAT(watcher_state_2->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(
-                  "", MakeCertKeyPairs(kIdentityCert2PrivateKey,
-                                       kIdentityCert2Contents))));
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  nullptr, MakeCertKeyPairs(kIdentityCert2PrivateKey,
+                                            kIdentityCert2Contents))));
   watcher_state_2->cert_update_queue.clear();
   // Cancel watcher 2.
   CancelWatch(watcher_state_2);
@@ -460,34 +524,36 @@ TEST_F(GrpcTlsCertificateDistributorTest,
   // Push credential updates to kCertName1 and check if the status works as
   // expected.
   distributor_.SetKeyMaterials(
-      kCertName1, kRootCert1Contents,
+      kCertName1, GetRootCert1(),
       MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents));
   // Check the updates are delivered to watcher 1.
   EXPECT_THAT(watcher_state_1->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(
-                  "", MakeCertKeyPairs(kIdentityCert1PrivateKey,
-                                       kIdentityCert1Contents))));
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  nullptr, MakeCertKeyPairs(kIdentityCert1PrivateKey,
+                                            kIdentityCert1Contents))));
   // Check the updates are delivered to watcher 2.
   EXPECT_THAT(watcher_state_2->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(kRootCert1Contents, {})));
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  EqRootCert(kRootCert1Contents), PemKeyCertPairList())));
   // Push root cert updates to kCertName1.
-  distributor_.SetKeyMaterials(kCertName1, kRootCert2Contents, std::nullopt);
+  distributor_.SetKeyMaterials(kCertName1, GetRootCert2(), std::nullopt);
   // Check the updates are delivered to watcher 2.
   EXPECT_THAT(watcher_state_2->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(kRootCert2Contents, {})));
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  EqRootCert(kRootCert2Contents), PemKeyCertPairList())));
   // Check the updates are not delivered to watcher 1.
   EXPECT_THAT(watcher_state_1->GetCredentialQueue(), ::testing::ElementsAre());
   // Push identity cert updates to kCertName1.
   distributor_.SetKeyMaterials(
-      kCertName1, std::nullopt,
+      kCertName1, nullptr,
       MakeCertKeyPairs(kIdentityCert2PrivateKey, kIdentityCert2Contents));
   // Check the updates are not delivered to watcher 2.
   EXPECT_THAT(watcher_state_2->GetCredentialQueue(), ::testing::ElementsAre());
   // Check the updates are delivered to watcher 1.
   EXPECT_THAT(watcher_state_1->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(
-                  "", MakeCertKeyPairs(kIdentityCert2PrivateKey,
-                                       kIdentityCert2Contents))));
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  nullptr, MakeCertKeyPairs(kIdentityCert2PrivateKey,
+                                            kIdentityCert2Contents))));
   // Cancel watcher 2.
   CancelWatch(watcher_state_2);
   EXPECT_THAT(GetCallbackQueue(),
@@ -509,7 +575,7 @@ TEST_F(GrpcTlsCertificateDistributorTest,
   EXPECT_THAT(GetCallbackQueue(), ::testing::ElementsAre());
   // Push credential updates to kCertName1.
   distributor_.SetKeyMaterials(
-      kCertName1, kRootCert1Contents,
+      kCertName1, GetRootCert1(),
       MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents));
   // Cancel watcher 2.
   CancelWatch(watcher_state_2);
@@ -524,13 +590,13 @@ TEST_F(GrpcTlsCertificateDistributorTest,
               ::testing::ElementsAre(CallbackStatus(kCertName1, true, true)));
   // Push credential updates to kCertName1.
   distributor_.SetKeyMaterials(
-      kCertName1, kRootCert2Contents,
+      kCertName1, GetRootCert2(),
       MakeCertKeyPairs(kIdentityCert2PrivateKey, kIdentityCert2Contents));
   // Check the updates are delivered to watcher 3.
   EXPECT_THAT(
       watcher_state_3->GetCredentialQueue(),
-      ::testing::ElementsAre(CredentialInfo(
-          kRootCert2Contents,
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(kRootCert2Contents),
           MakeCertKeyPairs(kIdentityCert2PrivateKey, kIdentityCert2Contents))));
   // Cancel watcher 3.
   CancelWatch(watcher_state_3);
@@ -555,17 +621,17 @@ TEST_F(GrpcTlsCertificateDistributorTest, SetKeyMaterialsInCallback) {
                                              bool /*root_being_watched*/,
                                              bool /*identity_being_watched*/) {
     distributor_.SetKeyMaterials(
-        cert_name, kRootCert1Contents,
+        cert_name, GetRootCert1(),
         MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents));
   });
   auto verify_function = [this](std::string cert_name) {
     WatcherState* watcher_state_1 = MakeWatcher(cert_name, cert_name);
     // Check the updates are delivered to watcher 1.
-    EXPECT_THAT(
-        watcher_state_1->GetCredentialQueue(),
-        ::testing::ElementsAre(CredentialInfo(
-            kRootCert1Contents, MakeCertKeyPairs(kIdentityCert1PrivateKey,
-                                                 kIdentityCert1Contents))));
+    EXPECT_THAT(watcher_state_1->GetCredentialQueue(),
+                ::testing::ElementsAre(MatchesCredentialInfo(
+                    EqRootCert(kRootCert1Contents),
+                    MakeCertKeyPairs(kIdentityCert1PrivateKey,
+                                     kIdentityCert1Contents))));
     CancelWatch(watcher_state_1);
   };
   // Start 10 threads that will register a watcher to a new cert name, verify
@@ -584,36 +650,36 @@ TEST_F(GrpcTlsCertificateDistributorTest, SetKeyMaterialsInCallback) {
 TEST_F(GrpcTlsCertificateDistributorTest, WatchACertInfoWithValidCredentials) {
   // Push credential updates to kCertName1.
   distributor_.SetKeyMaterials(
-      kCertName1, kRootCert1Contents,
+      kCertName1, GetRootCert1(),
       MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents));
   // Push root credential updates to kCertName2.
-  distributor_.SetKeyMaterials(kRootCert2Name, kRootCert2Contents,
-                               std::nullopt);
+  distributor_.SetKeyMaterials(kRootCert2Name, GetRootCert2(), std::nullopt);
   // Push identity credential updates to kCertName2.
   distributor_.SetKeyMaterials(
-      kIdentityCert2Name, std::nullopt,
+      kIdentityCert2Name, nullptr,
       MakeCertKeyPairs(kIdentityCert2PrivateKey, kIdentityCert2Contents));
   // Register watcher 1.
   WatcherState* watcher_state_1 = MakeWatcher(kCertName1, kCertName1);
   // watcher 1 should receive the credentials right away.
   EXPECT_THAT(
       watcher_state_1->GetCredentialQueue(),
-      ::testing::ElementsAre(CredentialInfo(
-          kRootCert1Contents,
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(kRootCert1Contents),
           MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents))));
   CancelWatch(watcher_state_1);
   // Register watcher 2.
   WatcherState* watcher_state_2 = MakeWatcher(kRootCert2Name, std::nullopt);
   // watcher 2 should receive the root credentials right away.
   EXPECT_THAT(watcher_state_2->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(kRootCert2Contents, {})));
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  EqRootCert(kRootCert2Contents), PemKeyCertPairList())));
   // Register watcher 3.
   WatcherState* watcher_state_3 = MakeWatcher(std::nullopt, kIdentityCert2Name);
   // watcher 3 should received the identity credentials right away.
   EXPECT_THAT(watcher_state_3->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(
-                  "", MakeCertKeyPairs(kIdentityCert2PrivateKey,
-                                       kIdentityCert2Contents))));
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  nullptr, MakeCertKeyPairs(kIdentityCert2PrivateKey,
+                                            kIdentityCert2Contents))));
   CancelWatch(watcher_state_2);
   CancelWatch(watcher_state_3);
 }
@@ -798,7 +864,7 @@ TEST_F(GrpcTlsCertificateDistributorTest,
        WatchErroredCertInfoWithValidCredentialData) {
   // Push credential updates to kCertName1.
   distributor_.SetKeyMaterials(
-      kCertName1, kRootCert1Contents,
+      kCertName1, GetRootCert1(),
       MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents));
   // Calling SetErrorForCert on both cert names.
   distributor_.SetErrorForCert(kCertName1, GRPC_ERROR_CREATE(kRootErrorMessage),
@@ -808,8 +874,8 @@ TEST_F(GrpcTlsCertificateDistributorTest,
   // watcher 1 should receive both the old credentials and the error right away.
   EXPECT_THAT(
       watcher_state_1->GetCredentialQueue(),
-      ::testing::ElementsAre(CredentialInfo(
-          kRootCert1Contents,
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(kRootCert1Contents),
           MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents))));
   EXPECT_THAT(watcher_state_1->GetErrorQueue(),
               ::testing::ElementsAre(
@@ -824,7 +890,7 @@ TEST_F(GrpcTlsCertificateDistributorTest,
                                GRPC_ERROR_CREATE(kIdentityErrorMessage));
   // Push credential updates to kCertName1.
   distributor_.SetKeyMaterials(
-      kCertName1, kRootCert1Contents,
+      kCertName1, GetRootCert1(),
       MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents));
   // Register watcher 1.
   WatcherState* watcher_state_1 = MakeWatcher(kCertName1, kCertName1);
@@ -832,8 +898,8 @@ TEST_F(GrpcTlsCertificateDistributorTest,
   // the previous error is wiped out by a successful update.
   EXPECT_THAT(
       watcher_state_1->GetCredentialQueue(),
-      ::testing::ElementsAre(CredentialInfo(
-          kRootCert1Contents,
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(kRootCert1Contents),
           MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents))));
   EXPECT_THAT(watcher_state_1->GetErrorQueue(), ::testing::ElementsAre());
   CancelWatch(watcher_state_1);
@@ -908,8 +974,117 @@ TEST_F(GrpcTlsCertificateDistributorTest, SetErrorForCertInCallback) {
   }
 }
 
-}  // namespace testing
+TEST_F(GrpcTlsCertificateDistributorTest,
+       BasicCredentialBehaviorsWithSpiffeRoot) {
+  EXPECT_FALSE(distributor_.HasRootCerts(kRootCert1Name));
+  EXPECT_FALSE(distributor_.HasKeyCertPairs(kIdentityCert1Name));
+  // After setting the certificates to the corresponding cert names, the
+  // distributor should possess the corresponding certs.
+  distributor_.SetKeyMaterials(kRootCert1Name, GetTestSpiffeBundleMap(),
+                               std::nullopt);
+  EXPECT_TRUE(distributor_.HasRootCerts(kRootCert1Name));
+  distributor_.SetKeyMaterials(
+      kIdentityCert1Name, nullptr,
+      MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents));
+  EXPECT_TRUE(distributor_.HasKeyCertPairs(kIdentityCert1Name));
+  // Querying a non-existing cert name should return false.
+  EXPECT_FALSE(distributor_.HasRootCerts(kRootCert2Name));
+  EXPECT_FALSE(distributor_.HasKeyCertPairs(kIdentityCert2Name));
+}
 
+TEST_F(GrpcTlsCertificateDistributorTest, UpdateCredentialsSpiffe) {
+  WatcherState* watcher_state_1 = MakeWatcher(kCertName1, kCertName1);
+  EXPECT_THAT(GetCallbackQueue(),
+              ::testing::ElementsAre(CallbackStatus(kCertName1, true, true)));
+  // SetKeyMaterials should trigger watcher's OnCertificatesChanged method.
+  distributor_.SetKeyMaterials(
+      kCertName1, GetTestSpiffeBundleMap(),
+      MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents));
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqSpiffeBundleMap(GetRawSpiffeBundleMap()),
+          MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents))));
+  // Set root certs should trigger watcher's OnCertificatesChanged again.
+  distributor_.SetKeyMaterials(kCertName1, GetTestSpiffeBundleMap2(),
+                               std::nullopt);
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqSpiffeBundleMap(GetRawSpiffeBundleMap2()),
+          MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents))));
+  // Set identity certs should trigger watcher's OnCertificatesChanged again.
+  distributor_.SetKeyMaterials(
+      kCertName1, nullptr,
+      MakeCertKeyPairs(kIdentityCert2PrivateKey, kIdentityCert2Contents));
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqSpiffeBundleMap(GetRawSpiffeBundleMap2()),
+          MakeCertKeyPairs(kIdentityCert2PrivateKey, kIdentityCert2Contents))));
+  CancelWatch(watcher_state_1);
+}
+
+TEST_F(
+    GrpcTlsCertificateDistributorTest,
+    AddAndCancelFirstWatcherForIdentityCertNameWithRootBeingWatchedWithSpiffe) {
+  // Register watcher 1 watching kCertName1 for root certs.
+  WatcherState* watcher_state_1 = MakeWatcher(kCertName1, std::nullopt);
+  EXPECT_THAT(GetCallbackQueue(),
+              ::testing::ElementsAre(CallbackStatus(kCertName1, true, false)));
+  // Register watcher 2 watching kCertName1 for identity certs.
+  WatcherState* watcher_state_2 = MakeWatcher(std::nullopt, kCertName1);
+  EXPECT_THAT(GetCallbackQueue(),
+              ::testing::ElementsAre(CallbackStatus(kCertName1, true, true)));
+  // Push credential updates to kCertName1 and check if the status works as
+  // expected.
+  distributor_.SetKeyMaterials(
+      kCertName1, GetTestSpiffeBundleMap(),
+      MakeCertKeyPairs(kIdentityCert1PrivateKey, kIdentityCert1Contents));
+  // Check the updates are delivered to watcher 1.
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqSpiffeBundleMap(GetRawSpiffeBundleMap()), PemKeyCertPairList())));
+  // Check the updates are delivered to watcher 2.
+  EXPECT_THAT(watcher_state_2->GetCredentialQueue(),
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  nullptr, MakeCertKeyPairs(kIdentityCert1PrivateKey,
+                                            kIdentityCert1Contents))));
+  // Push root cert updates to kCertName1.
+  distributor_.SetKeyMaterials(kCertName1, GetTestSpiffeBundleMap2(),
+                               std::nullopt);
+  // Check the updates are delivered to watcher 1.
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqSpiffeBundleMap(GetRawSpiffeBundleMap2()), PemKeyCertPairList())));
+  // Check the updates are not delivered to watcher 2.
+  EXPECT_THAT(watcher_state_2->GetCredentialQueue(), ::testing::ElementsAre());
+  // Push identity cert updates to kCertName1.
+  distributor_.SetKeyMaterials(
+      kCertName1, nullptr,
+      MakeCertKeyPairs(kIdentityCert2PrivateKey, kIdentityCert2Contents));
+  // Check the updates are not delivered to watcher 1.
+  EXPECT_THAT(watcher_state_1->GetCredentialQueue(), ::testing::ElementsAre());
+  // Check the updates are delivered to watcher 2.
+  EXPECT_THAT(watcher_state_2->GetCredentialQueue(),
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  nullptr, MakeCertKeyPairs(kIdentityCert2PrivateKey,
+                                            kIdentityCert2Contents))));
+  watcher_state_2->cert_update_queue.clear();
+  // Cancel watcher 2.
+  CancelWatch(watcher_state_2);
+  EXPECT_THAT(GetCallbackQueue(),
+              ::testing::ElementsAre(CallbackStatus(kCertName1, true, false)));
+  // Cancel watcher 1.
+  CancelWatch(watcher_state_1);
+  EXPECT_THAT(GetCallbackQueue(),
+              ::testing::ElementsAre(CallbackStatus(kCertName1, false, false)));
+}
+
+}  // namespace
+}  // namespace testing
 }  // namespace grpc_core
 
 int main(int argc, char** argv) {
