@@ -38,7 +38,7 @@ namespace {
 // returning status.
 CORE_END2END_TEST(Http2SingleHopTests, KeepaliveTimeout) {
   // Disable ping ack to trigger the keepalive timeout
-  InitServer(ChannelArgs().Set("grpc.http2.ack_pings", false));
+  InitServer(DefaultServerArgs().Set("grpc.http2.ack_pings", false));
   InitClient(ChannelArgs()
                  .Set(GRPC_ARG_KEEPALIVE_TIME_MS, 10)
                  .Set(GRPC_ARG_KEEPALIVE_TIMEOUT_MS, 0)
@@ -71,7 +71,7 @@ CORE_END2END_TEST(Http2SingleHopTests, ReadDelaysKeepalive) {
 #endif  // GRPC_POSIX_SOCKET
   const auto kPingInterval = Duration::Milliseconds(100);
   // Disable ping ack to trigger the keepalive timeout
-  InitServer(ChannelArgs().Set("grpc.http2.ack_pings", false));
+  InitServer(DefaultServerArgs().Set("grpc.http2.ack_pings", false));
   InitClient(ChannelArgs()
                  .Set(GRPC_ARG_KEEPALIVE_TIME_MS, (20 * kPingInterval).millis())
                  .Set(GRPC_ARG_KEEPALIVE_TIMEOUT_MS, 0)
