@@ -36,7 +36,7 @@ namespace {
 // - second attempt succeeds
 CORE_END2END_TEST(RetryTests, RetryServerPushbackDelay) {
   if (!IsRetryInCallv3Enabled()) SKIP_IF_V3();
-  InitServer(ChannelArgs());
+  InitServer(DefaultServerArgs());
   InitClient(ChannelArgs().Set(
       GRPC_ARG_SERVICE_CONFIG,
       "{\n"
