@@ -17,8 +17,8 @@
 //
 
 #include <grpc/grpc.h>
-#include <gtest/gtest.h>
 
+#include "gtest/gtest.h"
 #include "src/core/lib/iomgr/port.h"
 #include "src/core/util/crash.h"
 #include "test/core/test_util/test_config.h"
