@@ -39,7 +39,6 @@
 #include <variant>
 #include <vector>
 
-#include "absl/base/thread_annotations.h"
 #include "absl/functional/any_invocable.h"
 #include "absl/log/check.h"
 #include "absl/log/log.h"
@@ -57,6 +56,7 @@
 #include "src/core/client_channel/subchannel_pool_interface.h"
 #include "src/core/config/core_configuration.h"
 #include "src/core/credentials/call/call_credentials.h"
+#include "src/core/credentials/transport/transport_credentials.h"
 #include "src/core/lib/address_utils/parse_address.h"
 #include "src/core/lib/address_utils/sockaddr_utils.h"
 #include "src/core/lib/channel/channel_args.h"
@@ -102,6 +102,8 @@ class LoadBalancingPolicyTest : public ::testing::Test {
   using CallAttributes =
       std::vector<ServiceConfigCallData::CallAttributeInterface*>;
 
+  using RequestConnectionCallback = absl::AnyInvocable<void(absl::string_view)>;
+
   // Channel-level subchannel state for a specific address and channel args.
   // This is analogous to the real subchannel in the ClientChannel code.
   class SubchannelState {
@@ -116,7 +118,6 @@ class LoadBalancingPolicyTest : public ::testing::Test {
 
       ~FakeSubchannel() override {
         if (orca_watcher_ != nullptr) {
-          MutexLock lock(&state_->backend_metric_watcher_mu_);
           state_->orca_watchers_.erase(orca_watcher_.get());
         }
         for (const auto& p : watcher_map_) {
@@ -172,8 +173,7 @@ class LoadBalancingPolicyTest : public ::testing::Test {
       void WatchConnectivityState(
           std::unique_ptr<
               SubchannelInterface::ConnectivityStateWatcherInterface>
-              watcher) override
-          ABSL_EXCLUSIVE_LOCKS_REQUIRED(*state_->test_->work_serializer_) {
+              watcher) override {
         auto* watcher_ptr = watcher.get();
         auto watcher_wrapper = MakeOrphanable<WatcherWrapper>(
             state_->work_serializer(), std::move(watcher));
@@ -183,8 +183,7 @@ class LoadBalancingPolicyTest : public ::testing::Test {
       }
 
       void CancelConnectivityStateWatch(
-          ConnectivityStateWatcherInterface* watcher) override
-          ABSL_EXCLUSIVE_LOCKS_REQUIRED(*state_->test_->work_serializer_) {
+          ConnectivityStateWatcherInterface* watcher) override {
         auto it = watcher_map_.find(watcher);
         if (it == watcher_map_.end()) return;
         state_->state_tracker_.RemoveWatcher(it->second);
@@ -192,14 +191,15 @@ class LoadBalancingPolicyTest : public ::testing::Test {
       }
 
       void RequestConnection() override {
-        MutexLock lock(&state_->requested_connection_mu_);
+        LOG(INFO) << "LB policy requested connection for " << state_->address_;
         state_->requested_connection_ = true;
+        if (state_->test_->request_connection_callback_ != nullptr) {
+          state_->test_->request_connection_callback_(state_->address_);
+        }
       }
 
       void AddDataWatcher(
-          std::unique_ptr<DataWatcherInterface> watcher) override
-          ABSL_EXCLUSIVE_LOCKS_REQUIRED(*state_->test_->work_serializer_) {
-        MutexLock lock(&state_->backend_metric_watcher_mu_);
+          std::unique_ptr<DataWatcherInterface> watcher) override {
         auto* w =
             static_cast<InternalSubchannelDataWatcherInterface*>(watcher.get());
         if (w->type() == OrcaProducer::Type()) {
@@ -226,9 +226,7 @@ class LoadBalancingPolicyTest : public ::testing::Test {
         }
       }
 
-      void CancelDataWatcher(DataWatcherInterface* watcher) override
-          ABSL_EXCLUSIVE_LOCKS_REQUIRED(*state_->test_->work_serializer_) {
-        MutexLock lock(&state_->backend_metric_watcher_mu_);
+      void CancelDataWatcher(DataWatcherInterface* watcher) override {
         auto* w = static_cast<InternalSubchannelDataWatcherInterface*>(watcher);
         if (w->type() == OrcaProducer::Type()) {
           if (orca_watcher_.get() != static_cast<OrcaWatcher*>(watcher)) return;
@@ -312,6 +310,8 @@ class LoadBalancingPolicyTest : public ::testing::Test {
         bool validate_state_transition = true,
         absl::AnyInvocable<void()> run_before_flush = nullptr,
         SourceLocation location = SourceLocation()) {
+      LOG(INFO) << "Setting connectivity state of " << address_ << " to "
+                << ConnectivityStateName(state) << " (" << status << ")";
       ExecCtx exec_ctx;
       if (state == GRPC_CHANNEL_TRANSIENT_FAILURE) {
         EXPECT_FALSE(status.ok())
@@ -328,30 +328,29 @@ class LoadBalancingPolicyTest : public ::testing::Test {
       // scheduled on the WorkSerializer.  We don't want to return until
       // all of those notifications have been delivered.
       absl::Notification notification;
-      test_->work_serializer_->Run(
-          [&]() ABSL_EXCLUSIVE_LOCKS_REQUIRED(*test_->work_serializer_) {
-            if (validate_state_transition) {
-              AssertValidConnectivityStateTransition(state_tracker_.state(),
-                                                     state, location);
-            }
-            LOG(INFO) << "Setting state on tracker";
-            state_tracker_.SetState(state, status, "set from test");
-            // SetState() enqueued the connectivity state notifications for
-            // the subchannel, so we add another callback to the queue to be
-            // executed after that state notifications has been delivered.
-            if (run_before_flush != nullptr) run_before_flush();
-            LOG(INFO) << "Waiting for state notifications to be delivered";
-            test_->work_serializer_->Run([&]() {
-              LOG(INFO) << "State notifications delivered, waiting for "
-                           "health notifications";
-              // Now the connectivity state notifications has been
-              // delivered. If the state reported was READY, then the
-              // pick_first leaf policy will have started a health watch, so
-              // we add another callback to the queue to be executed after
-              // the initial health watch notification has been delivered.
-              test_->work_serializer_->Run([&]() { notification.Notify(); });
-            });
-          });
+      test_->work_serializer_->Run([&]() {
+        if (validate_state_transition) {
+          AssertValidConnectivityStateTransition(state_tracker_.state(), state,
+                                                 location);
+        }
+        LOG(INFO) << "Setting state on tracker";
+        state_tracker_.SetState(state, status, "set from test");
+        // SetState() enqueued the connectivity state notifications for
+        // the subchannel, so we add another callback to the queue to be
+        // executed after that state notifications has been delivered.
+        if (run_before_flush != nullptr) run_before_flush();
+        LOG(INFO) << "Waiting for state notifications to be delivered";
+        test_->work_serializer_->Run([&]() {
+          LOG(INFO) << "State notifications delivered, waiting for "
+                       "health notifications";
+          // Now the connectivity state notifications has been
+          // delivered. If the state reported was READY, then the
+          // pick_first leaf policy will have started a health watch, so
+          // we add another callback to the queue to be executed after
+          // the initial health watch notification has been delivered.
+          test_->work_serializer_->Run([&]() { notification.Notify(); });
+        });
+      });
       while (!notification.HasBeenNotified()) {
         test_->fuzzing_ee_->Tick();
       }
@@ -362,7 +361,6 @@ class LoadBalancingPolicyTest : public ::testing::Test {
     // have requested a connection attempt since the last time this
     // method was called.
     bool ConnectionRequested() {
-      MutexLock lock(&requested_connection_mu_);
       return std::exchange(requested_connection_, false);
     }
 
@@ -373,7 +371,6 @@ class LoadBalancingPolicyTest : public ::testing::Test {
 
     // Sends an OOB backend metric report to all watchers.
     void SendOobBackendMetricReport(const BackendMetricData& backend_metrics) {
-      MutexLock lock(&backend_metric_watcher_mu_);
       for (const auto* watcher : orca_watchers_) {
         watcher->watcher()->OnBackendMetricReport(backend_metrics);
       }
@@ -382,7 +379,6 @@ class LoadBalancingPolicyTest : public ::testing::Test {
     // Checks that all OOB watchers have the expected reporting period.
     void CheckOobReportingPeriod(Duration expected,
                                  SourceLocation location = SourceLocation()) {
-      MutexLock lock(&backend_metric_watcher_mu_);
       for (const auto* watcher : orca_watchers_) {
         EXPECT_EQ(watcher->report_interval(), expected)
             << location.file() << ":" << location.line();
@@ -392,11 +388,10 @@ class LoadBalancingPolicyTest : public ::testing::Test {
     size_t NumWatchers() const {
       size_t num_watchers;
       absl::Notification notification;
-      work_serializer()->Run(
-          [&]() ABSL_EXCLUSIVE_LOCKS_REQUIRED(*test_->work_serializer_) {
-            num_watchers = state_tracker_.NumWatchers();
-            notification.Notify();
-          });
+      work_serializer()->Run([&]() {
+        num_watchers = state_tracker_.NumWatchers();
+        notification.Notify();
+      });
       while (!notification.HasBeenNotified()) {
         test_->fuzzing_ee_->Tick();
       }
@@ -412,16 +407,11 @@ class LoadBalancingPolicyTest : public ::testing::Test {
    private:
     const std::string address_;
     LoadBalancingPolicyTest* const test_;
-    ConnectivityStateTracker state_tracker_
-        ABSL_GUARDED_BY(*test_->work_serializer_);
+    ConnectivityStateTracker state_tracker_;
 
-    Mutex requested_connection_mu_;
-    bool requested_connection_ ABSL_GUARDED_BY(&requested_connection_mu_) =
-        false;
+    bool requested_connection_ = false;
 
-    Mutex backend_metric_watcher_mu_;
-    std::set<OrcaWatcher*> orca_watchers_
-        ABSL_GUARDED_BY(&backend_metric_watcher_mu_);
+    std::set<OrcaWatcher*> orca_watchers_;
   };
 
   // A fake helper to be passed to the LB policy.
@@ -447,15 +437,11 @@ class LoadBalancingPolicyTest : public ::testing::Test {
 
     explicit FakeHelper(LoadBalancingPolicyTest* test) : test_(test) {}
 
-    bool QueueEmpty() {
-      MutexLock lock(&mu_);
-      return queue_.empty();
-    }
+    bool QueueEmpty() { return queue_.empty(); }
 
     // Called at test tear-down time to ensure that we have not left any
     // unexpected events in the queue.
     void ExpectQueueEmpty(SourceLocation location = SourceLocation()) {
-      MutexLock lock(&mu_);
       EXPECT_TRUE(queue_.empty())
           << location.file() << ":" << location.line() << "\n"
           << QueueString();
@@ -467,7 +453,6 @@ class LoadBalancingPolicyTest : public ::testing::Test {
     // the queue.
     std::optional<StateUpdate> GetNextStateUpdate(
         SourceLocation location = SourceLocation()) {
-      MutexLock lock(&mu_);
       EXPECT_FALSE(queue_.empty()) << location.file() << ":" << location.line();
       if (queue_.empty()) return std::nullopt;
       Event& event = queue_.front();
@@ -488,7 +473,6 @@ class LoadBalancingPolicyTest : public ::testing::Test {
     // from the queue.
     std::optional<ReresolutionRequested> GetNextReresolution(
         SourceLocation location = SourceLocation()) {
-      MutexLock lock(&mu_);
       EXPECT_FALSE(queue_.empty()) << location.file() << ":" << location.line();
       if (queue_.empty()) return std::nullopt;
       Event& event = queue_.front();
@@ -515,7 +499,7 @@ class LoadBalancingPolicyTest : public ::testing::Test {
           });
     }
 
-    std::string QueueString() const ABSL_EXCLUSIVE_LOCKS_REQUIRED(&mu_) {
+    std::string QueueString() const {
       std::vector<std::string> parts = {"Queue:"};
       for (const Event& event : queue_) {
         parts.push_back(EventString(event));
@@ -545,7 +529,6 @@ class LoadBalancingPolicyTest : public ::testing::Test {
     void UpdateState(
         grpc_connectivity_state state, const absl::Status& status,
         RefCountedPtr<LoadBalancingPolicy::SubchannelPicker> picker) override {
-      MutexLock lock(&mu_);
       StateUpdate update{state, status, std::move(picker)};
       LOG(INFO) << "enqueuing state update from LB policy: "
                 << update.ToString();
@@ -553,7 +536,6 @@ class LoadBalancingPolicyTest : public ::testing::Test {
     }
 
     void RequestReresolution() override {
-      MutexLock lock(&mu_);
       queue_.push_back(ReresolutionRequested());
     }
 
@@ -577,12 +559,11 @@ class LoadBalancingPolicyTest : public ::testing::Test {
       return test_->stats_plugin_group_;
     }
 
-    void AddTraceEvent(TraceSeverity, absl::string_view) override {}
+    void AddTraceEvent(absl::string_view) override {}
 
     LoadBalancingPolicyTest* test_;
 
-    Mutex mu_;
-    std::deque<Event> queue_ ABSL_GUARDED_BY(&mu_);
+    std::deque<Event> queue_;
   };
 
   // A fake MetadataInterface implementation, for use in PickArgs.
@@ -633,7 +614,8 @@ class LoadBalancingPolicyTest : public ::testing::Test {
       return nullptr;
     }
 
-    ClientCallTracer::CallAttemptTracer* GetCallAttemptTracer() const override {
+    ClientCallTracerInterface::CallAttemptTracer* GetCallAttemptTracer()
+        const override {
       return nullptr;
     }
 
@@ -955,51 +937,6 @@ class LoadBalancingPolicyTest : public ::testing::Test {
     return retval;
   }
 
-  // Waits for the round_robin policy to start using an updated address list.
-  // There can be any number of READY updates where the picker is still using
-  // the old list followed by one READY update where the picker is using the
-  // new list.  Returns a picker if the reported states match expectations.
-  RefCountedPtr<LoadBalancingPolicy::SubchannelPicker>
-  WaitForRoundRobinListChange(absl::Span<const absl::string_view> old_addresses,
-                              absl::Span<const absl::string_view> new_addresses,
-                              const CallAttributes& call_attributes = {},
-                              size_t num_iterations = 3,
-                              SourceLocation location = SourceLocation()) {
-    LOG(INFO) << "Waiting for expected RR addresses...";
-    RefCountedPtr<LoadBalancingPolicy::SubchannelPicker> retval;
-    size_t num_picks =
-        std::max(new_addresses.size(), old_addresses.size()) * num_iterations;
-    WaitForStateUpdate(
-        [&](FakeHelper::StateUpdate update) {
-          EXPECT_EQ(update.state, GRPC_CHANNEL_READY)
-              << location.file() << ":" << location.line();
-          if (update.state != GRPC_CHANNEL_READY) return false;
-          // Get enough picks to round-robin num_iterations times across all
-          // expected addresses.
-          auto picks = GetCompletePicks(update.picker.get(), num_picks,
-                                        call_attributes, nullptr, location);
-          EXPECT_TRUE(picks.has_value())
-              << location.file() << ":" << location.line();
-          if (!picks.has_value()) return false;
-          LOG(INFO) << "PICKS: " << absl::StrJoin(*picks, " ");
-          // If the picks still match the old list, then keep going.
-          if (PicksAreRoundRobin(old_addresses, *picks)) return true;
-          // Otherwise, the picks should match the new list.
-          bool matches = PicksAreRoundRobin(new_addresses, *picks);
-          EXPECT_TRUE(matches)
-              << "Expected: " << absl::StrJoin(new_addresses, ", ")
-              << "\nActual: " << absl::StrJoin(*picks, ", ") << "\nat "
-              << location.file() << ":" << location.line();
-          if (matches) {
-            retval = std::move(update.picker);
-          }
-          return false;  // Stop.
-        },
-        location);
-    LOG(INFO) << "done waiting for expected RR addresses";
-    return retval;
-  }
-
   // Expects a state update for the specified state and status, and then
   // expects the resulting picker to queue picks.
   bool ExpectStateAndQueuingPicker(
@@ -1133,6 +1070,51 @@ class LoadBalancingPolicyTest : public ::testing::Test {
     return results;
   }
 
+  // Waits for the round_robin policy to start using an updated address list.
+  // There can be any number of READY updates where the picker is still using
+  // the old list followed by one READY update where the picker is using the
+  // new list.  Returns a picker if the reported states match expectations.
+  RefCountedPtr<LoadBalancingPolicy::SubchannelPicker>
+  WaitForRoundRobinListChange(absl::Span<const absl::string_view> old_addresses,
+                              absl::Span<const absl::string_view> new_addresses,
+                              const CallAttributes& call_attributes = {},
+                              size_t num_iterations = 3,
+                              SourceLocation location = SourceLocation()) {
+    LOG(INFO) << "Waiting for expected RR addresses...";
+    RefCountedPtr<LoadBalancingPolicy::SubchannelPicker> retval;
+    size_t num_picks =
+        std::max(new_addresses.size(), old_addresses.size()) * num_iterations;
+    WaitForStateUpdate(
+        [&](FakeHelper::StateUpdate update) {
+          EXPECT_EQ(update.state, GRPC_CHANNEL_READY)
+              << location.file() << ":" << location.line();
+          if (update.state != GRPC_CHANNEL_READY) return false;
+          // Get enough picks to round-robin num_iterations times across all
+          // expected addresses.
+          auto picks = GetCompletePicks(update.picker.get(), num_picks,
+                                        call_attributes, nullptr, location);
+          EXPECT_TRUE(picks.has_value())
+              << location.file() << ":" << location.line();
+          if (!picks.has_value()) return false;
+          LOG(INFO) << "PICKS: " << absl::StrJoin(*picks, " ");
+          // If the picks still match the old list, then keep going.
+          if (PicksAreRoundRobin(old_addresses, *picks)) return true;
+          // Otherwise, the picks should match the new list.
+          bool matches = PicksAreRoundRobin(new_addresses, *picks);
+          EXPECT_TRUE(matches)
+              << "Expected: " << absl::StrJoin(new_addresses, ", ")
+              << "\nActual: " << absl::StrJoin(*picks, ", ") << "\nat "
+              << location.file() << ":" << location.line();
+          if (matches) {
+            retval = std::move(update.picker);
+          }
+          return false;  // Stop.
+        },
+        location);
+    LOG(INFO) << "done waiting for expected RR addresses";
+    return retval;
+  }
+
   // Returns true if the list of actual pick result addresses matches the
   // list of expected addresses for round_robin.  Note that the actual
   // addresses may start anywhere in the list of expected addresses but
@@ -1442,6 +1424,7 @@ class LoadBalancingPolicyTest : public ::testing::Test {
   GlobalStatsPluginRegistry::StatsPluginGroup stats_plugin_group_;
   std::string target_ = "dns:server.example.com";
   std::string authority_ = "server.example.com";
+  RequestConnectionCallback request_connection_callback_;
 };
 
 }  // namespace testing
