@@ -30,6 +30,7 @@
 #include "gmock/gmock.h"
 #include "gtest/gtest.h"
 #include "src/core/config/core_configuration.h"
+#include "src/core/ext/transport/chttp2/transport/http2_settings.h"
 #include "src/core/ext/transport/chttp2/transport/http2_status.h"
 #include "src/core/ext/transport/chttp2/transport/transport_common.h"
 #include "src/core/lib/channel/channel_args.h"
@@ -99,8 +100,7 @@ TEST_F(Http2ClientTransportTest, TestHttp2ClientTransportObjectCreation) {
       {
           EventEngineSlice(
               grpc_slice_from_copied_string(GRPC_CHTTP2_CLIENT_CONNECT_STRING)),
-          helper_.EventEngineSliceFromHttp2SettingsFrame(
-              {{4, helper_.GetDefaultInitialWindowSize()}}),
+          helper_.EventEngineSliceFromHttp2SettingsFrameDefault(),
       },
       event_engine().get());
   mock_endpoint.ExpectRead(
@@ -142,8 +142,7 @@ TEST_F(Http2ClientTransportTest, TestHttp2ClientTransportWriteFromQueue) {
       {
           EventEngineSlice(
               grpc_slice_from_copied_string(GRPC_CHTTP2_CLIENT_CONNECT_STRING)),
-          helper_.EventEngineSliceFromHttp2SettingsFrame(
-              {{4, helper_.GetDefaultInitialWindowSize()}}),
+          helper_.EventEngineSliceFromHttp2SettingsFrameDefault(),
       },
       event_engine().get());
   mock_endpoint.ExpectWrite(
@@ -190,8 +189,7 @@ TEST_F(Http2ClientTransportTest, TestHttp2ClientTransportWriteFromCall) {
       {
           EventEngineSlice(
               grpc_slice_from_copied_string(GRPC_CHTTP2_CLIENT_CONNECT_STRING)),
-          helper_.EventEngineSliceFromHttp2SettingsFrame(
-              {{4, helper_.GetDefaultInitialWindowSize()}}),
+          helper_.EventEngineSliceFromHttp2SettingsFrameDefault(),
 
       },
       event_engine().get());
@@ -257,8 +255,7 @@ TEST_F(Http2ClientTransportTest, Http2ClientTransportAbortTest) {
       {
           EventEngineSlice(
               grpc_slice_from_copied_string(GRPC_CHTTP2_CLIENT_CONNECT_STRING)),
-          helper_.EventEngineSliceFromHttp2SettingsFrame(
-              {{4, helper_.GetDefaultInitialWindowSize()}}),
+          helper_.EventEngineSliceFromHttp2SettingsFrameDefault(),
       },
       event_engine().get());
   mock_endpoint.ExpectWrite(
@@ -313,8 +310,7 @@ TEST_F(Http2ClientTransportTest, TestHttp2ClientTransportPingRead) {
       {
           EventEngineSlice(
               grpc_slice_from_copied_string(GRPC_CHTTP2_CLIENT_CONNECT_STRING)),
-          helper_.EventEngineSliceFromHttp2SettingsFrame(
-              {{4, helper_.GetDefaultInitialWindowSize()}}),
+          helper_.EventEngineSliceFromHttp2SettingsFrameDefault(),
       },
       event_engine().get());
 
@@ -366,8 +362,7 @@ TEST_F(Http2ClientTransportTest, TestHttp2ClientTransportPingWrite) {
       {
           EventEngineSlice(
               grpc_slice_from_copied_string(GRPC_CHTTP2_CLIENT_CONNECT_STRING)),
-          helper_.EventEngineSliceFromHttp2SettingsFrame(
-              {{4, helper_.GetDefaultInitialWindowSize()}}),
+          helper_.EventEngineSliceFromHttp2SettingsFrameDefault(),
       },
       event_engine().get());
   mock_endpoint.ExpectWriteWithCallback(
@@ -440,8 +435,7 @@ TEST_F(Http2ClientTransportTest, TestHttp2ClientTransportPingTimeout) {
       {
           EventEngineSlice(
               grpc_slice_from_copied_string(GRPC_CHTTP2_CLIENT_CONNECT_STRING)),
-          helper_.EventEngineSliceFromHttp2SettingsFrame(
-              {{4, helper_.GetDefaultInitialWindowSize()}}),
+          helper_.EventEngineSliceFromHttp2SettingsFrameDefault(),
       },
       event_engine().get());
   mock_endpoint.ExpectWriteWithCallback(
@@ -499,8 +493,7 @@ TEST_F(Http2ClientTransportTest, TestHttp2ClientTransportMultiplePings) {
       {
           EventEngineSlice(
               grpc_slice_from_copied_string(GRPC_CHTTP2_CLIENT_CONNECT_STRING)),
-          helper_.EventEngineSliceFromHttp2SettingsFrame(
-              {{4, helper_.GetDefaultInitialWindowSize()}}),
+          helper_.EventEngineSliceFromHttp2SettingsFrameDefault(),
       },
       event_engine().get());
   mock_endpoint.ExpectWriteWithCallback(
@@ -599,8 +592,7 @@ TEST_F(Http2ClientTransportTest, TestHeaderDataHeaderFrameOrder) {
       {
           EventEngineSlice(
               grpc_slice_from_copied_string(GRPC_CHTTP2_CLIENT_CONNECT_STRING)),
-          helper_.EventEngineSliceFromHttp2SettingsFrame(
-              {{4, helper_.GetDefaultInitialWindowSize()}}),
+          helper_.EventEngineSliceFromHttp2SettingsFrameDefault(),
       },
       event_engine().get());
   mock_endpoint.ExpectWrite(
@@ -692,6 +684,62 @@ TEST_F(Http2ClientTransportTest, TestHeaderDataHeaderFrameOrder) {
   event_engine()->UnsetGlobalHooks();
 }
 
+TEST(Http2CommonTransportTest, TestReadChannelArgs) {
+  // Test to validate that ReadChannelArgs reads all the channel args
+  // correctly.
+  Http2Settings settings;
+  ChannelArgs channel_args =
+      ChannelArgs()
+          .Set(GRPC_ARG_HTTP2_HPACK_TABLE_SIZE_DECODER, 2048)
+          .Set(GRPC_ARG_HTTP2_STREAM_LOOKAHEAD_BYTES, 1024)
+          .Set(GRPC_ARG_HTTP2_MAX_FRAME_SIZE, 16384)
+          .Set(GRPC_ARG_EXPERIMENTAL_HTTP2_PREFERRED_CRYPTO_FRAME_SIZE, true)
+          .Set(GRPC_ARG_HTTP2_ENABLE_TRUE_BINARY, 1)
+          .Set(GRPC_ARG_SECURITY_FRAME_ALLOWED, true);
+  ReadSettingsFromChannelArgs(channel_args, settings, /*is_client=*/true);
+  // Settings read from ChannelArgs.
+  EXPECT_EQ(settings.header_table_size(), 2048u);
+  EXPECT_EQ(settings.initial_window_size(), 1024u);
+  EXPECT_EQ(settings.max_frame_size(), 16384u);
+  EXPECT_EQ(settings.preferred_receive_crypto_message_size(), INT_MAX);
+  EXPECT_EQ(settings.allow_true_binary_metadata(), true);
+  EXPECT_EQ(settings.allow_security_frame(), true);
+  // Default settings
+  EXPECT_EQ(settings.max_concurrent_streams(), 4294967295u);
+  EXPECT_EQ(settings.max_header_list_size(), 16384u);
+  EXPECT_EQ(settings.enable_push(), true);
+
+  // If ChannelArgs don't have a value for the setting, the default must be
+  // loaded into the Settings object
+  Http2Settings settings2;
+  EXPECT_EQ(settings2.header_table_size(), 4096u);
+  EXPECT_EQ(settings2.max_concurrent_streams(), 4294967295u);
+  EXPECT_EQ(settings2.initial_window_size(), 65535u);
+  EXPECT_EQ(settings2.max_frame_size(), 16384u);
+  // TODO(tjagtap) : [PH2][P4] : Investigate why we change it in
+  // ReadSettingsFromChannelArgs . Right now ReadSettingsFromChannelArgs is
+  // functinally similar to the legacy read_channel_args.
+  EXPECT_EQ(settings2.max_header_list_size(), 16777216u);
+  EXPECT_EQ(settings2.preferred_receive_crypto_message_size(), 0u);
+  EXPECT_EQ(settings2.enable_push(), true);
+  EXPECT_EQ(settings2.allow_true_binary_metadata(), false);
+  EXPECT_EQ(settings2.allow_security_frame(), false);
+
+  ReadSettingsFromChannelArgs(ChannelArgs(), settings2, /*is_client=*/true);
+  EXPECT_EQ(settings2.header_table_size(), 4096u);
+  EXPECT_EQ(settings2.max_concurrent_streams(), 4294967295u);
+  EXPECT_EQ(settings2.initial_window_size(), 65535u);
+  EXPECT_EQ(settings2.max_frame_size(), 16384u);
+  // TODO(tjagtap) : [PH2][P4] : Investigate why we change it in
+  // ReadSettingsFromChannelArgs . Right now ReadSettingsFromChannelArgs is
+  // functinally similar to the legacy read_channel_args.
+  EXPECT_EQ(settings2.max_header_list_size(), 16384u);
+  EXPECT_EQ(settings2.preferred_receive_crypto_message_size(), 0u);
+  EXPECT_EQ(settings2.enable_push(), true);
+  EXPECT_EQ(settings2.allow_true_binary_metadata(), false);
+  EXPECT_EQ(settings2.allow_security_frame(), false);
+}
+
 // TODO(tjagtap) : [PH2][P2] Write tests similar to
 // TestHeaderDataHeaderFrameOrder for Continuation frame read.
 
